import os, sys
from cookieutils.key import CKey
import time
import cookieutils

if os.name=='nt': # windows
  from pynput import keyboard
else: #unix
  import tty,termios

operating_system = cookieutils.get_os()



def __presets(key):
    map = {
        "backspace": CKey.backspace,
        "enter":  CKey.enter,
        "space":  CKey.space,
        "tab":   CKey.tab,
        "esc":  CKey.esc,
        "up":  CKey.up,
        "down":  CKey.down,
        "right":  CKey.right,
        "left":  CKey.left
    }
    if key.name in map.keys():
        return map[key.name]
    else:
        return False


def __getkey():
    old_settings = termios.tcgetattr(sys.stdin)
    tty.setcbreak(sys.stdin.fileno())
    try:
        while True:
            b = os.read(sys.stdin.fileno(), 3).decode()
            if len(b) == 3:
                k = ord(b[2])
            else:
                k = ord(b)
            key_mapping = {
                127: CKey.backspace,
                10:  CKey.enter,
                32:  CKey.space,
                9:   CKey.tab,
                27:  CKey.esc,
                65:  CKey.up,
                66:  CKey.down,
                67:  CKey.right,
                68:  CKey.left
            }
            return key_mapping.get(k, chr(k))
    finally:
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, old_settings)


def get():
    if operating_system == "windows":
        time.sleep(0.3) # gives it some time so it does not pick up missinputs, also prevents it from imidently shutting off for some reason lol idk why
        with keyboard.Events() as events: # if on a windows system
            event = events.get()
            if event is None:
                raise ValueError('Uh oh. somthing just went wrong with the event. contact the dev lol, CODE: 001')
            else:
                if event.__class__ == keyboard.Events.Press:
                    try:
                        return {"type":"letter", "data":event.key.char}
                    except AttributeError: # if it is a letter or not
                        return {"type":"key", "data":__presets(event.key)}


    elif operating_system == "unix": # if on a unix system
        try:
            k = __getkey()
            try:
                if k.name:
                    return {"type":"key", "data":k}
            except AttributeError: # check if it is a letter or not
                    return {"type":"letter", "data":str(k)}
        except (KeyboardInterrupt):
            os.system('stty sane')


def color(text:str, color:int=15, bgcolor:int=None):
    if bgcolor:
        return(f"\033[48;5;{bgcolor}m\033[38;5;{color}m{text}\033[0;0m")
    else:
        return(f"\033[38;5;{str(color)}m{str(text)}\033[0;0m")




def rgb_to_xterm(rgb:tuple):
    r, g, b = rgb
    N = []
    for i, n in enumerate([47, 68, 40, 40, 40, 21]):
        N.extend([i]*n)
    mx = max(r, g, b)    
    mn = min(r, g, b)

    if (mx-mn)*(mx+mn) <= 6250:
        c = 24 - (252 - ((r+g+b) // 3)) // 10
        if 0 <= c <= 23:
            
            return 232 + c

    return 16 + 36*N[r] + 6*N[g] + N[b]

def hex_to_xterm(hex):

    if hex.startswith("#"):
        hex = hex.lstrip('#')

    return rgb_to_xterm(tuple(int(hex[i:i+2], 16) for i in (0, 2, 4)))



# -*- coding: utf-8 -*-
__version__ = "0.0.3"
import os
if os.name=='nt': # windows
  __operating_system__ = "windows"
else: #unix
  __operating_system__ = "unix"


def get_version():
    return __version__

def get_os():
    return __operating_system__


import uuid as uuid_modual
import secrets
import string
import random


def hex(length):
    return secrets.token_hex(length)

def uuid():
    return uuid_modual.uuid4()

def custom(length, low=False,up=True,numbers=True,symbols=False,custom:list=None):
    if not custom:
        chars = []
        if low:
            chars = chars + list(string.ascii_lowercase)
        if up:
            chars = chars + list(string.ascii_uppercase)
        if numbers:
            chars = chars + list(string.digits)
        if symbols:
            chars = chars + list(string.punctuation)
    else:
        chars = custom
    return "".join([random.choice(chars) for x in range(length)])


import clear
import codes
import colors
from cookieutils import key
import keys


input("Press enter to oscls")
clear.oscls()
input("Press enter to pcls")
clear.pcls()

input("Run codes")
print(codes.hex(10))
print(codes.uuid())
print(codes.custom(20, True, True, True, True))
print(codes.custom(20, custom=["<",">"]))

input("run colors")
print(colors.color("xterm", 124, 19))
print(colors.color("both",  colors.hex_to_xterm("#f41c18"), colors.rgb_to_xterm((24, 50, 244))))

input("run key")
print("press g")
if keys.get()["data"] == "g":
    print("YES")
else:
    quit("NO!")
print("press backspace")
if keys.get()["data"] == key.CKey.backspace:
    print("YES")
else:
    quit("NO!")


import enum
class CKey():
    """A class representing various buttons that may not correspond to
    letters. This includes modifier keys and function keys.

    The actual values for these items differ between platforms. Some platforms
    may have additional buttons, but these are guaranteed to be present
    everywhere.

    this is a modifyed version of the one from the keyboard modual. It has removed keys for capatibilty
    
    """
    

    #: The Backspace key.
    class backspace():
        name = "backspace"
    #: A down arrow key.
    class down():
        name = "down"
    #: The Enter or Return key.
    class enter():
        name = "enter"
    #: The Esc key.
    class esc():
        name = "esc"
    #: A left arrow key.
    class left():
        name = "left"
    #: A right arrow key.
    class right():
        name = "right"
    #: The Space key.
    class space():
        name = "space"

    #: The Tab key.
    class tab():
        name = "tab"

    #: An up arrow key.
    class up():
        name = "up"



